function [out_image error]=bsm_main(NITER, img, proj_angles, loud, video, initial_temperature, in_guess, NOISE)
%
% Simulated annealing to solve binary tomography resonstructions.
% For parameters, se bsm.m
% load an input image, X0 and especially the projection matrix M and the
% projections: c
% Written at SSIP 2008 in Vienna
% last modified Mittwoch 9/7 by Erik Wernersson


%Calculate the projections with last years method.

% calculate the center of the image
%c=round(0.5*sqrt(numel(img)));
for a=proj_angles
    [A b]=projectionmatrix(img,a,[25 25],50);
    if exist('M')
        M=[M;A];
    else 
        M=A;
    end
    if exist('c')
        c=[c;b];
    else
        c=b;
    end
end
[t1 t2]=size(c);
c = c+NOISE*randn(t1,t2);

% If already pre-defined matrixes, load them here.
%M=load('noise/M_cross_rot_noise');
%M=M.M_cross_rot_noise;
%c=load('noise/c_cross_rot_noise');
%c=c.c_cross_rot_noise;

%[M c]=bsm_hvproj(img); %Calculate vertical and horizontal projections

[nx ny]=size(img); % Get the resolution

if loud==1
figure(1)  % Open main figure window
subplot(2,3,1) % Display the original image here
imagesc(img)
title('original image')
colormap('gray')
end

%%%% create an initial guess, X0 if no one already exists
if in_guess==0 % if no initial guess was provided
% This should make use of information in c
X0=zeros(numel(img),1);
nproj=2; %number of projections
% put ones in the initial guess until the number of elements mathches the
% projections that is trusted.
while sum(X0)<sum(c)/numel(proj_angles)
    pos=ceil(rand(1,1)*numel(X0)-1)+1;
    X0(pos)=1;
end
else
    disp('The last guess from the previous run is used');
    X0=in_guess(:);
end % end of initial guess initialization



if loud==1
subplot(2,3,2) % Show the inital guess in the main window
imagesc(reshape(X0,ny,nx))
title('Initial Guess')
colormap('gray')
end

%%% Simulated annealing
if loud ==1
    disp('Starting simulated annealing') %
end


% Own method:
t=initial_temperature;
sv=zeros(NITER,1); % A vector to store the error in
temps=zeros(NITER,1); %A vector to store the temperatures in

XN = proposeNewImage(X0,t); % Make a initial suggestion
current=bsm_objectiveFunction(M,X0,c); % Evaluate the error

delta_max=0; % the maximal change in energy between iterations
for tv=1:NITER % main loop
t=bsm_newTemp(tv,NITER);
XN = bsm_proposeNewImage(X0,t); % Make a suggestion for the next picture
proposed=bsm_objectiveFunction(M,XN,c);
if abs(proposed-current)>delta_max
    delta_max=abs(proposed-current);
end
if proposed<current % If the proposed is _equal_ or better, chose that one.
    X0=XN;
    current=proposed;
else
if exp(10/delta_max*(current-proposed)/t)>rand(1)
    X0=XN;
    current=proposed;
end

end
sv(tv)=current;
temps(tv)=t;
if current==0 % Break the loop if no error
    break
end
if video
    if mod(tv,1000)==0
    figure(4)
    imagesc(reshape(X0,ny,nx))
    colormap(gray)
    Mov(tv/1000)=getframe();
    end
end
end

if loud==1
figure(1) % Show the errors along the iterations
subplot(2,3,4)
plot(1:tv,sv(1:tv))
title('||Ax-b||')
ylabel('Error/Energy')
xlabel('Iteration')
subplot(2,3,5)
plot(1:tv, temps);
ylabel('T');
xlabel('Iteration')
disp(strcat('Iteration ended after:',int2str(tv),' iterations. The total error was: ' , int2str(current)))
end
error=sv(tv); % The final error
out_image=reshape(X0,ny,nx);
figure(1) % Display the final result
subplot(2,3,3)
imagesc(out_image)
title('Result of simulated annealing')
colormap('gray')



% fin

if video
    disp('Playing movie...');
    figure(5)
    movie(Mov)
end

end    